/**
 * Title:        GridSim Toolkit
 * Version:      GridSim  1.0, 12/09/2001
 * Description:  GridSim (Grid Simulation) Toolkit for Modeling and Simulation of
 *               Parallel and Distributed Systems such as Clusters and Grids
 * @author       R. Buyya and M. Murshed (by surname)
 *               Monash University, Melbourne, Australia
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 */

import eduni.simjava.*;
import gridsim.*;

/**
 * The User Experiment paramter class.
 */
public class Experiment
{
  private int id;
  private GridletList glList;
  private double B_factor;  // Budget Factor
  private double D_factor;  // Deadline Factor
  private double deadline; // calculated based on D_factor
  private double budget; // calculated based on B_factor
  private double StartTime; // set by user or current time
  private double EndTime; // EndTime <= StartTime + deadline
  private int OptimizationStrategy;

  public static final int OPTIMIZE_COST = 0; // Cost minimization policy.
  public static final int OPTIMIZE_COST_PLUS = 1; // First aim for Cost minimization, but powerful machines first if same cost.
  public static final int OPTIMIZE_COST_TIME = 2; // First aim for Cost minimization and Time Optimisation if possible.
  public static final int OPTIMIZE_TIME = 3; // Time minimization policy.
  public static final int OPTIMIZE_NONE = 4; // None minimization policy.

  public Experiment(int id, GridletList glList,
                    int OptimizationStrategy, double B_factor, double D_factor)
  {
    this.id = id;
    this.glList = glList;
    this.StartTime = Sim_system.clock();
    this.B_factor = B_factor;
    this.D_factor = D_factor;
    this.OptimizationStrategy = OptimizationStrategy;
  }

  public int GetID() {
    return id;
  }

  public GridletList GetGridletList() {
    return glList;
  }

  public int GetOptimizationStrategy() {
    return OptimizationStrategy;
  }
  // Returns the Gridlet status as ASCII text (e.g., "Created")
  public String GetOptimizationStrategyName()
  {
    String nameString = new String();
    switch( GetOptimizationStrategy() )
    {
      case OPTIMIZE_COST:
           nameString = "OPTIMIZE_COST";
           break;
      case OPTIMIZE_COST_PLUS:
           nameString = "OPTIMIZE_COST_PLUS";
           break;
      case OPTIMIZE_COST_TIME:
           nameString = "OPTIMIZE_COST_TIME";
           break;
      case OPTIMIZE_TIME:
           nameString = "OPTIMIZE_TIME";
           break;
      case OPTIMIZE_NONE:
           nameString = "OPTIMIZE_NONE";
           break;
    }
    return nameString;
  }

  public double GetB_factor() {
    return B_factor;
  }

  public double GetD_factor() {
    return D_factor;
  }

  public void SetDeadline(double deadline) {
    this.deadline = deadline;
  }

  public double GetDeadline() {
    return deadline;
  }

  public void SetBudget(double budget) {
    this.budget = budget;
  }

  public double GetBudget() {
    return budget;
  }

  public void SetEndTime() {
    this.EndTime = Sim_system.clock();
  }

  public double GetStartTime() {
    return StartTime;
  }

  public double GetEndTime() {
    return EndTime;
  }

  public int GetNoOfGridlets() {
    return glList.size();
  }

  public double GetBudgetSpent() {
    double expenses = 0;
    for(int i = 0; i < GetNoOfGridlets(); i++)
    {
      if(((Gridlet) glList.get(i)).GetStatus()==Gridlet.SUCCESS)
        expenses += ((Gridlet) glList.get(i)).GetActualCPUTime()*((Gridlet) glList.get(i)).GetCostPerSec();
    }
    return expenses;
  }

  public int GetNoOfSuccessfulGridlets() {
    int count = 0;
    for(int i = 0; i < GetNoOfGridlets(); i++)
      if(((Gridlet) glList.get(i)).GetStatus()==Gridlet.SUCCESS)
        count++;
    return count;
  }

  public boolean IsSuccessful() {
    if(GetNoOfSuccessfulGridlets()==GetNoOfGridlets())
      return true;
    else
      return false;
  }

  public double GridletCompletionRate() {
    return ((double) GetNoOfSuccessfulGridlets())/GetNoOfGridlets();
  }

  public double TimeUtilization() {
    return (EndTime - StartTime) / (deadline - StartTime);
  }

  public double BudgetUtilization() {
    return GetBudgetSpent() / budget;
  }

  public Accumulator GetAccLength() { // An accumulator of the length of all the gridlets
    Accumulator accLength = new Accumulator();
    for(int i = 0; i < GetNoOfGridlets(); i++)
      accLength.add(((Gridlet) glList.get(i)).GetGridletLength());
    return accLength;
  }
}
