/**
 * Title:        Grid Broker
 * Version:      1.0, 12/09/2001
 * Description:  A Grid Scheduler for Application Scheduling on Grid based on
 *               Deadline and Budget Constrained Scheduling Algorithms
 * Authors:      Rajkumar Buyya and Manzur Murshed
 *               Monash University, Melbourne, Australia
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 */

import java.util.*;
import java.io.*;
import eduni.simjava.*;
import gridsim.*;

/**
 * Grid User Entity, which creates Grid Environment, setups experiment and invokes the broker.
 */
class UserEntity extends GridSim
{
  private String BrokerName;        // Name of the Broker Entity
  private Experiment eSend;         // Experiment for Sumission with Gridlet to be processed
  private Experiment eReceived;     // Experiment Processed
  private Random r_delay;
  private double how_long;
  private double SuccessiveExperimentDelay;
  private Accumulator GridletCompletionRate;
  private Accumulator TimeUtilization;
  private Accumulator BudgetUtilization;

  public UserEntity(String name, Experiment expt, double baud_rate, double how_long, long seed, double SuccessiveExperimentDelay)
  {
    super(name);

    r_delay = new Random(seed*31);
    this.how_long = how_long;

    // Create a Broker with entity name as "Broker_UserEntityName"
    BrokerName = new String("Broker_"+name);
    new Broker(BrokerName, baud_rate);
    eSend = expt;
    this.SuccessiveExperimentDelay = SuccessiveExperimentDelay;
    GridletCompletionRate = new Accumulator();
    TimeUtilization = new Accumulator();
    BudgetUtilization = new Accumulator();
  }

  public void body()
  {
    RecordStatistics("USER.B_factor", eSend.GetB_factor());
    RecordStatistics("USER.D_factor", eSend.GetD_factor());

//    while(Sim_system.clock() < how_long)
    {
      // Wait
//      sim_hold(GridSimRandom.real(SuccessiveExperimentDelay, 0.1, 0.1, r_delay.nextDouble()));

//      System.out.println(Sim_system.clock()+"\t"+get_name());
//      System.out.println(get_name()+" Sending Experiment: "+ExperimentCount+" to the broker");

      //Send Experiment to the Broker.
      Send(BrokerName, GridSimTags.SCHEDULE_NOW, GridSimTags.EXPERIMENT, eSend );

      // Wait for Experiment Completion event
      eReceived = (Experiment) ReceiveEventObject();
      // set current simulation time as expt. completion time
      eReceived.SetEndTime();

      GridletCompletionRate.add(eReceived.GridletCompletionRate());
      TimeUtilization.add(eReceived.TimeUtilization());
      BudgetUtilization.add(eReceived.BudgetUtilization());

      RecordStatistics("USER.Experiment.No", eReceived.GetID());
      RecordStatistics("USER.Experiment.StartTime", eReceived.GetStartTime());
      RecordStatistics("USER.Experiment.EndTime", eReceived.GetEndTime());
      RecordStatistics("USER.Experiment.GridletCompletionRate", eReceived.GridletCompletionRate());
      RecordStatistics("USER.Experiment.TimeUtilization", eReceived.TimeUtilization());
      RecordStatistics("USER.Experiment.BudgetUtilization", eReceived.BudgetUtilization());

      // Gridlet execution data
      System.out.println(get_name()+":: Exeperiment Schedule"+"\n-----------------------------------");
      System.out.println("UserID | GridLetID | Status | Resource ID | GL_StartTime | Gl_FinishTime | Resource Name");
      for( int i = 0; i < eSend.GetGridletList().size(); i++)
      {
        Gridlet gl = (Gridlet)(eSend.GetGridletList().get(i));
        String res_name = (String) Sim_system.get_entity(gl.GetResourceID()).get_name();
        System.out.println(get_name() +" | "+ gl.GetGridletID()+" | "+ gl.GetStatusString()+" | "+gl.GetResourceID()+" | "+
        gl.getExecStartTime() +" | "+ gl.GetFinishTime() +" | "+ res_name);
      }

      // Exeperimet Statistics
      System.out.println("******** Experiment Report ********");
      System.out.println("No. of Gridlets: "+eReceived.GetNoOfGridlets());
      System.out.println("Deadline Factor: "+eReceived.GetD_factor());
      System.out.println("Deadline: "+eReceived.GetDeadline());
      System.out.println("Budget Factor: "+eReceived.GetB_factor());
      System.out.println("Budget: "+eReceived.GetBudget());
      System.out.println("Stratergy: "+eReceived.GetOptimizationStrategyName());
      System.out.println("Start Time: "+eReceived.GetStartTime());
      System.out.println("End Time: "+eReceived.GetEndTime());
      System.out.println("Budget Spent: "+eReceived.GetBudgetSpent());
      System.out.println("Gridlets Executed: "+eReceived.GetNoOfSuccessfulGridlets());
      System.out.println("Gridlet Completion Rate: "+eReceived.GridletCompletionRate());

    }

    RecordStatistics("*.USER.GridletCompletionRate", GridletCompletionRate.mean());
    RecordStatistics("*.USER.TimeUtilization", TimeUtilization.mean());
    System.out.println(get_name()+" TU: "+ TimeUtilization.mean());
    RecordStatistics("*.USER.BudgetUtilization", BudgetUtilization.mean());

    System.out.println("GridletCompletionRate: "+(int)(GridletCompletionRate.mean()*100) +"\t"+
                      "TimeUtilization: "+ (int)(TimeUtilization.mean()*100) +"\t"+
                      "BudgetUtilization: "+(int)(BudgetUtilization.mean()*100));

    // Signal the END to broker.
    Send(BrokerName, GridSimTags.SCHEDULE_NOW, GridSimTags.END_OF_SIMULATION );

    // Signal the this User's End to GridSim: This should be invoked at the end.
    ShutdownUserEntity();
  }
}

class RunConfiguration
{
  String configFile;
  public int no_of_users;
  public int users_sample_points;
  public int no_of_resources;
  public int resource_sample_points;

  public RunConfiguration( String filename )
  {
    configFile = filename;
  }

  public boolean ReadConfig()
  {
    try {
      FileInputStream InFileStream = new FileInputStream(configFile);
      DataInputStream cf = new DataInputStream(InFileStream);

      String conf;
      conf = cf.readLine();
      StringTokenizer st = new StringTokenizer(conf);

      Integer I;
      I = new Integer((String)st.nextElement());
      no_of_users = I.intValue();

      I = new Integer((String)st.nextElement());
      users_sample_points = I.intValue();

      I = new Integer((String)st.nextElement());
      no_of_resources = I.intValue();

      I = new Integer((String)st.nextElement());
      resource_sample_points = I.intValue();

      cf.close();
      InFileStream.close();
    }

    catch(FileNotFoundException e)
    {
      System.out.println("Unable to Open File for Reading: "+configFile);
      return false;
    }
    catch(IOException e)
    {
      System.out.println("File reading error");
      return false;
    }
    return true;
  }

  /** read String from the keyboard until ENTER key is pressed.
   *  @return It returns String read from the keyboard.
   */
  public static String readString()
  {
    String text = "";
    int num = 0;
    // read until ENTER key is pressed.
    while(true)
    {
      try
      {
        num = System.in.read();
      }
      catch(IOException e)
      {}
      if( (char) num == '\n' )  // if Carriage Return is pressed, then end.
         break;
      text = text+(char)num;  // concatanate character
    }
    return text;
  }
}

class GridSimMain
{
  public static void main(String[] args)
  {
    int no_of_users = 1;
    int no_of_resources = 10;
    double B_factor = 0.5;  // this value can vary between 0.1 to 1.0, higher value means higher budget
    double D_factor = 0.5;
    // scheduling policy can Experiment.OPTIMIZE_COST || OPTIMIZE_COST_PLUS || OPTIMIZE_COST_TIME ...
    int policy = Experiment.OPTIMIZE_COST;
    double how_long = 2.0*60*60; // 2 hours of simulation time
    String reportFile = new String("1");

    // Example Run1: Deadline and Budget Constrained (DBC) with Cost Optimisation Schedulung
    // Resource Cost Per Second (G$/sec.)
    int resCosts[] = { 10, 12, 14, 16, 18, 20, 22, 24, 26, 28};
    policy = Experiment.OPTIMIZE_COST;
    SimulateAMS2000Grid(no_of_users, no_of_resources, resCosts, B_factor, D_factor, policy, how_long, reportFile);


    // Example Run2: Deadline and Budget Constrained Schedulung with Cost-Time Optimisation
    // Time Optimisation is performed as long as it does not costs exatra expenses.
    // e.g., 3 cheapest resources with the same cost.
    // OPTIMIZE_COST_TIME policy can take advantage of this situation
    int resCosts1[] = { 10, 12, 14, 16, 18, 20, 22, 24, 10, 10};
    policy = Experiment.OPTIMIZE_COST_TIME;
    SimulateAMS2000Grid(no_of_users, no_of_resources, resCosts1, B_factor, D_factor, policy, how_long, reportFile);

    // Example Run3: Deadline and Budget Constrained Schedulung with Cost Optimisation
    // for different Deadline and Budget Constraints
    policy = Experiment.OPTIMIZE_COST;
    for( D_factor = 0.1; D_factor <= 1.0; D_factor += 0.1)
      for( B_factor = 0.1; B_factor <= 1.0; B_factor += 0.1)
        SimulateAMS2000Grid(no_of_users, no_of_resources, resCosts, B_factor, D_factor, policy, how_long, reportFile);

    System.out.println("Congratulations!");
  }

  // GridSim EXAMPLE1: All resources with the same capability but different cost.
  public static void SimulateAMS2000Grid(int no_of_users, int no_of_resources, int resCosts[],
                double B_factor, double D_factor, int policy, double how_long, String ReportFile)
  {
    Calendar calender = Calendar.getInstance();

    long seed = 11L*13*17*19*23+1;  // What is Seed ? OR Why this big number
    seed = seed * 97 + 1;

//  System.out.println(calender.getTime());
    System.out.println("Users:"+no_of_users+", Machines:"+no_of_resources+", B:"+B_factor+", D:"+D_factor);

    String[] eff = {""};
    String[] efp = {""};
    String ReportWriterName = "ReportWriter1";

    GridSim.Init(no_of_users, calender, true, eff, efp, ReportWriterName);

    // Create Report Writer Entity
    new ReportWriter(ReportWriterName, no_of_users, no_of_resources, ReportFile);

    // Create Resources: ResourceCharacteristics = TIME_SHARED
    for(int i=0; i<no_of_resources; i++)
    {
      // create PEs
      PEList peList = new PEList();
      peList.add(new PE(0, (int)GridSimStandardPE.GetRating()));

      MachineList mList = new MachineList();
      mList.add(new Machine(0, peList, ResourceCharacteristics.TIME_SHARED));
      // create a resource containing machines
      ResourceCharacteristics r = new ResourceCharacteristics( "INTEL", "Linux",
            mList, ResourceCharacteristics.TIME_SHARED, 0.0, (double)resCosts[i]);
      LinkedList Weekends = new LinkedList();
      Weekends.add(new Integer(Calendar.SATURDAY));
      Weekends.add(new Integer(Calendar.SUNDAY));
      LinkedList Holidays = new LinkedList();
      new GridResource("Resource_"+i, 28000.0, seed*(i+1)+1, r, 0.0, 0.0, 0.0, Weekends, Holidays);
    }

    // Create Application, Experiment, and Users
    for(int i=0; i<no_of_users; i++)
    {
      GridletList glList = AMS2000_Application();
      Experiment expt = new Experiment(0, glList, policy, B_factor, D_factor);
      new UserEntity("User_"+i, expt, 28000.0, how_long, seed*997*(1+i)+1, 100);
    }
    // Start Simulation
    GridSim.Start();
  }

  public static GridletList AMS2000_Application()
  {
    // Gridlet creation
    GridletList glList = new GridletList();
    int GridletCount = 100;
    double GridletLength = GridSimStandardPE.toMIs(90);
    for(int i = 0; i < GridletCount; i++)
    {
      Gridlet gl = new Gridlet(i, GridletLength, 0, 0);
      glList.add(gl);
    }
    return glList;
  }


  // GridSim EXAMPLE2
  public static void ReadSimDataAndRun()
  {
    double B_factor = 0.5;
    double D_factor = 1.0;
    int policy = Experiment.OPTIMIZE_COST;
    double how_long = 2.0*60*60; // simulation time
    String configFile;
    String reportFile;
    RunConfiguration rc;

    System.out.print("Enter Name of Configuration File: ");
    configFile = RunConfiguration.readString();

    rc = new RunConfiguration(configFile);
    if( !rc.ReadConfig() )
    {
      System.out.println("Configuration file "+configFile+ " does not exist.");
      return;
    }

    System.out.print("Enter Name of Report File: ");
    reportFile = RunConfiguration.readString();

    int users_step_size = rc.no_of_users / rc.users_sample_points;
    int resources_step_size =  rc.no_of_resources / rc.resource_sample_points;

    for(int i = 0; i < rc.no_of_users; i += users_step_size)
      for(int j = 0; j < rc.no_of_resources; j += resources_step_size)
      {
        System.out.println("No of Users: "+rc.no_of_users +"\t"+"Resource: "+rc.no_of_resources +"\t"+
              "B Factor: "+B_factor +"\t"+"D Factor: "+D_factor);
        SimulateGrid(i+1, j+1, B_factor, D_factor, policy, how_long, reportFile);
      }
  }

  public static void SimulateGrid(int no_of_users, int no_of_resources,
                double B_factor, double D_factor, int policy, double how_long, String ReportFile)
  {
    Calendar calender = Calendar.getInstance();

    long seed = 11L*13*17*19*23+1;  // What is Seed ? OR Why this big number
    seed = seed * 97 + 1;

//  System.out.println(calender.getTime());
    System.out.println("Users:"+no_of_users+", Machines:"+no_of_resources+", B:"+B_factor+", D:"+D_factor);

//    GridSimRandom rand = new GridSimRandom(seed, 0.1, 0.1, 0.1, 01.);

    String[] eff = {""};
    String[] efp = {""};
    String ReportWriterName = "ReportWriter1";

    GridSim.Init(no_of_users, calender, true, eff, efp, ReportWriterName);

    // Create Report Writer Entity
    new ReportWriter(ReportWriterName, no_of_users, no_of_resources, ReportFile);

    // Create Resources: ResourceCharacteristics = TIME_SHARED
    for(int i=0; i<no_of_resources; i++)
    {
      // create PEs
      PEList peList = new PEList();
      for(int j=0; j<(i+1); j++)
         peList.add(new PE(j, GridSimStandardPE.GetRating()*(1+i)));

      // create machine list
      MachineList mList = new MachineList();
      mList.add(new Machine(0, peList, ResourceCharacteristics.TIME_SHARED));
      // create a resource containing machines
      ResourceCharacteristics r = new ResourceCharacteristics( "INTEL", "Linux",
            mList, ResourceCharacteristics.TIME_SHARED, 0.0, 1+i*1);
      LinkedList Weekends = new LinkedList();
      Weekends.add(new Integer(Calendar.SATURDAY));
      Weekends.add(new Integer(Calendar.SUNDAY));
      LinkedList Holidays = new LinkedList();
      new GridResource("Resource_"+i, 28000.0, seed*(i+1)+1, r, 0.0, 0.0, 0.0, Weekends, Holidays);
    }

    Random r = new Random(seed);
    // Create Application, Experiment, and Users
    for(int i=0; i<no_of_users; i++)
    {
      GridletList glList = Application1(r);
      Experiment expt = new Experiment(0, glList, policy, B_factor, D_factor);
      new UserEntity("User_"+i, expt, 28000.0, how_long, seed*997*(1+i)+1, 0);
    }
    // Start Simulation
    GridSim.Start();
  }

  public static GridletList Application1(Random r)
  {
    // Gridlet creation
    GridletList glList = new GridletList();
    int GridletCount = (int) GridSimRandom.real(20, 0.1, 0.1, r.nextDouble());
    double GridletLength = GridSimStandardPE.toMIs(GridSimRandom.real(10, 0.1, 0.1, r.nextDouble()));
    for(int i = 0; i < GridletCount; i++)
    {
      double glLength = GridSimRandom.real(GridletLength, 0.0, 0.2, r.nextDouble());
      Gridlet gl = new Gridlet(i, GridletLength, 1024, 2*1024);
      glList.add(gl);
    }
    return glList;
  }
}
