/**
 * Title:        GridSim Toolkit
 * Version:      GridSim  1.0, 12/09/2001
 * Description:  GridSim (Grid Simulation) Toolkit for Modeling and Simulation of
 *               Parallel and Distributed Systems such as Clusters and Grids
 * @author       R. Buyya and M. Murshed (by surname)
 *               Monash University, Melbourne, Australia
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 */

package gridsim;
import eduni.simjava.*;

/**
 * Classes related to Communication between User Entities and Resources
 */

public class IO_data
{
  private Object data;
  private long byte_size;
//  private int src_id; // should be used later for generating acknowledgement
  private int dest_id;

  public IO_data(Object data, long byte_size, int dest_id)
  {
    this.data = data;
    this.byte_size = byte_size;
//    this.src_id = src_id;
    this.dest_id = dest_id;
  }

  public Object get_data() {
    return data;
  }

  public long get_byte_size() {
    return byte_size;
  }

  public int get_dest_id() {
    return dest_id;
  }

//  public int get_src_id() {
//    return src_id;
//  }
}

/**
 * It accepts messages that comes from GridSim entities 'Output' entity
 * and passes the same to the GridSim entity.
 * It simulates Network communication delay depending on Baud rate
 * and data length.
 */
class Input extends Sim_entity
{
  private Sim_port in_port;
  private double baud_rate;

  public Input(String name, double baud_rate) {
    super(name);
    this.baud_rate = baud_rate;
    in_port = new Sim_port("input_buffer");
    add_port(in_port);
  }

  public double GetBaudRate() {
    return baud_rate;
  }

  public double real_io(double d) {
    return GridSimRandom.real_io(d);
  }

  public void body()
  {
    // Process events
    Sim_event ev = new Sim_event();
    for(sim_get_next(ev); ev.get_tag()!=GridSimTags.END_OF_SIMULATION; sim_get_next(ev))
    {
      if(ev.get_data()!=null)
      {
        IO_data io = (IO_data) ev.get_data();
        // Simulate Transmission Time after Receiving
// Hold first then dispatch
        double sender_baud_rate = ((Output) Sim_system.get_entity(ev.get_src())).GetBaudRate();
        double min_baud_rate = Math.min(baud_rate, sender_baud_rate);
        sim_hold(real_io((io.get_byte_size()/min_baud_rate))); // reading time

        // Deliver Event to the entity (its parent) to which it is acting as buffer
        sim_schedule(in_port, GridSimTags.SCHEDULE_NOW, ev.get_tag(), io.get_data());
//				System.out.println(get_name()+":  "+Sim_system.get_entity(ev.get_src()).get_name());
      }
    }
  }
}

/**
 * It sends messages to destination entity's Input entity.
 * It works along with Input entity to simulate network
 * communication delay.
 */
class Output extends Sim_entity
{
  private Sim_port out_port;
  private double baud_rate;

  public Output(String name, double baud_rate) {
    super(name);
    this.baud_rate = baud_rate;
    out_port = new Sim_port("output_buffer");
    add_port(out_port);
  }

  public double GetBaudRate() {
    return baud_rate;
  }

  public double real_io(double d)
  {
    return GridSimRandom.real_io(d);
  }

  public void body()
  {
    // Process events
    Sim_event ev = new Sim_event();
//    for(sim_get_next(new Sim_from_port(out_port), ev); ev.get_tag()!=GridSimTags.END_OF_SIMULATION; sim_get_next(new Sim_from_port(out_port), ev))
    for(sim_get_next(ev); ev.get_tag()!=GridSimTags.END_OF_SIMULATION; sim_get_next(ev))
    {
      if(ev.get_data()!=null)
      {
        IO_data io = (IO_data) ev.get_data();
        // Identify ID of an entity which acts as Input/Buffer entity of destination entity
        int id = GridSim.GetEntityId("Input_"+Sim_system.get_entity(io.get_dest_id()).get_name());
//				System.out.println(get_name()+":  "+Sim_system.get_entity(io.get_dest_id()).get_name());
        //  Send first and then hold
        sim_schedule(id, GridSimTags.SCHEDULE_NOW, ev.get_tag(), io);
        double receiver_baud_rate = ((Input) Sim_system.get_entity(id)).GetBaudRate();
        double min_baud_rate = Math.min(baud_rate, receiver_baud_rate);
        sim_hold(real_io(io.get_byte_size()/min_baud_rate));
      }
    }
  }
}
